
from enum import Enum
from evdev import ecodes, uinput

import random
import time

from autopilot.display import Display
from autopilot.input import Touch


class SwipeDirection(Enum):
    """Swipe a direction with simulated finger

    :param Enum: number for swipe gesture

    top_to_bottom = 1
    left_to_right = 2
    bottom_to_top = 3
    right_to_left = 4
    middle_to_left = 5
    middle_to_right = 6
    """

    top_to_bottom = 1
    left_to_right = 2
    bottom_to_top = 3
    right_to_left = 4
    middle_to_left = 5
    middle_to_right = 6

    @staticmethod
    def get_random_direction():
        return SwipeDirection.__members__.get(
            random.choice(list(SwipeDirection.__members__.keys()))
        )


def swipe(direction):
    device = Touch.create()
    display = Display.create()

    screen_width = display.get_screen_width()
    screen_height = display.get_screen_height()
    start_x = start_y = end_x = end_y = None
    if direction == SwipeDirection.top_to_bottom:
        start_x = end_x = screen_width / 2
        start_y, end_y = 0, screen_height
    elif direction == SwipeDirection.left_to_right:
        start_x, end_x = 0, screen_width
        start_y = end_y = screen_height / 2
    elif direction == SwipeDirection.bottom_to_top:
        start_x = end_x = screen_width / 2
        start_y, end_y = screen_height, 0
    elif direction == SwipeDirection.right_to_left:
        start_x, end_x = screen_width, 0
        start_y = end_y = screen_height / 2
    elif direction == SwipeDirection.middle_to_left:
        start_x, start_y = screen_width/2, screen_height/2
        end_x, end_y = 0, screen_height/2
    elif direction == SwipeDirection.middle_to_right:
        start_x, start_y = screen_width/2, screen_height/2
        end_x, end_y = screen_width, screen_height/2
    device.drag(start_x, start_y, end_x, end_y)


def click_password_input():
    display = Display.create()

    screen_width = display.get_screen_width()
    screen_height = display.get_screen_height()
    for i in range(76, 100):
        Touch.create().tap(int(screen_width / 2), int(screen_height / 2 - i))


def tap_button(key):
    """Tap a button using uinput

    :param key: the ecode

    >>>tap_button(e.KEY_POWER)  # taps the power button
    """
    with uinput.UInput() as ui:
        ui.write(ecodes.EV_KEY, key, 1)
        ui.write(ecodes.EV_KEY, key, 0)
        ui.syn()


def type_password():
    tap_button(ecodes.KEY_1)
    tap_button(ecodes.KEY_2)
    tap_button(ecodes.KEY_3)
    tap_button(ecodes.KEY_4)
    tap_button(ecodes.KEY_ENTER)


def unlock_ui():
    swipe(SwipeDirection(5))
    time.sleep(1)
    click_password_input()
    type_password()


def get_display_size():
    display = Display.create()

    return '{}x{}'.format(
        display.get_screen_width(),
        display.get_screen_height()
    )


def click_random_location():
    display = Display.create()

    screen_width = display.get_screen_width()
    screen_height = display.get_screen_height()
    x = random.randrange(0, int(screen_width))
    y = random.randrange(0, int(screen_height))

    Touch.create().tap(x, y)
