import json
import os
import subunit

from helpers import create_json_for_nfss

from testtools import TestCase


class InvalidArguments(Exception):
        pass


class TestDictionary:
    def __init__(self, dictionary):
        self.test_dict = dictionary
        self.details = dictionary['details']

test_dictionary = {
    'details': {
        'crash_file': ["/var/crash/lol.crash"],
        'battery_left': '11',
        'cpu_useage': '33',
        'codename': 'utopic',
        'release': '14.10',
        'free_mem': '23%',
        'serial_number': '1',
        'hardware_model': 'lol',
        'total_test_time': '5',
        'device_temp_during_crash': '100',
        'ubuntu_version': '9000',
        'device_version': '20140708',
        'test_name': 'random_gestures',
    }
}


class LRTCreateJsonTestCase(TestCase):
    def setUp(self):
        super(LRTCreateJsonTestCase, self).setUp()
        self.dict_for_test = test_dictionary
        self.local_path = os.path.dirname(os.path.abspath(__file__))
        self.subunit_file = os.path.join(
            self.local_path, 'data/result.subunit'
        )
        self.result_dict = create_json_for_nfss.BuildResultDict(
            self.subunit_file
        )

    def test_command_line_arg_file_option_missing(self):
        parser = create_json_for_nfss._create_parser()
        self.assertRaises(
            SystemExit,
            parser.parse_args,
            ['']
        )

    def test_command_line_arg_verbose_neg(self):
        parser = create_json_for_nfss._create_parser()
        args = parser.parse_args(['-f', self.local_path])
        self.assertFalse(args.verbose)

    def test_command_line_arg_verbose_short(self):
        parser = create_json_for_nfss._create_parser()
        args = parser.parse_args(args=['-v', '-f', self.local_path])
        self.assertTrue(args.verbose)

    def test_command_line_arg_verbose_long(self):
        parser = create_json_for_nfss._create_parser()
        args = parser.parse_args(args=['--verbose', '--file', self.local_path])
        self.assertTrue(args.verbose)

    def test_get_stream_result_is_stream_result(self):
        """get_stream_result returns stream result"""
        stream_result = create_json_for_nfss.get_stream_result(
            self.subunit_file
        )
        self.assertIsInstance(
            stream_result,
            subunit.v2.ByteStreamToStreamResult
        )

    def test_build_result_dict_details(self):
        """result_dict.details has expected dictionary keys"""
        self.assertIn('crash_file', self.result_dict.details)
        self.assertIn('ubuntu_version', self.result_dict.details)
        self.assertIn('device_version', self.result_dict.details)
        self.assertIn('hardware_model', self.result_dict.details)
        self.assertIn('serial_number', self.result_dict.details)
        self.assertIn('total_test_time', self.result_dict.details)
        self.assertIn('device_temp_during_crash', self.result_dict.details)
        self.assertIn('codename', self.result_dict.details)
        self.assertIn('release', self.result_dict.details)
        self.assertIn('cpu_useage', self.result_dict.details)
        self.assertIn('free_mem', self.result_dict.details)
        self.assertIn('battery_left', self.result_dict.details)
        self.assertIn('test_name', self.result_dict.details)

    def test_build_result_dict_test_dict(self):
        self.assertIs(dict, type(self.result_dict.test_dict))
        self.assertEqual('fail', self.result_dict.test_dict['status'])

    def test_get_detail_text(self):
        """get_detail returns text value of content"""
        self.assertEqual(
            self.result_dict.get_detail('serial_number'),
            u'04d709211598502a'
        )

    def test__missing_key(self):
        """Verify there are no missing keys"""
        dictionary = TestDictionary(self.dict_for_test)
        self.assertFalse(create_json_for_nfss._missing_key(dictionary))
        self.assertFalse(create_json_for_nfss._missing_key(self.result_dict))

    def test__missing_key_missing_serial_number(self):
        """Verify _missing_key returns true when serial_number is missing"""
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['serial_number']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test__missing_key_missing_hardware_model(self):
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['hardware_model']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test__missing_key_missing_total_test_time(self):
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['total_test_time']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test__missing_key_missing_device_temp_during_crash(self):
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['device_temp_during_crash']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test__missing_key_missing_ubuntu_version(self):
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['ubuntu_version']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test__missing_key_missing_device_version(self):
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['device_version']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test__missing_key_missing_cpu_useage(self):
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['cpu_useage']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test__missing_key_missing_free_mem(self):
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['free_mem']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test__missing_key_missing_release(self):
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['release']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test__missing_key_missing_codename(self):
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['codename']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test__missing_key_missing_test_name(self):
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['test_name']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test__missing_key_missing_crash_file(self):
        dictionary = TestDictionary(self.dict_for_test)
        del dictionary.details['crash_file']
        self.assertTrue(create_json_for_nfss._missing_key(dictionary))

    def test_create_json_returns_json(self):
        test_json = self.result_dict.create_json()
        self.assertTrue(json.loads(test_json))
